/*!  \file 

$Id: main.c,v 1.1 2003/04/5 18:22:25 tsp_admin Exp $

-----------------------------------------------------------------------

GDISP+ - Graphic Tool based on GTK+,
         for being used upon the generic Transport Sampling Protocol.

Copyright (c) 2003 - Yves DUFRENNE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-----------------------------------------------------------------------

Project   : TSP
Maintainer: tsp@astrium-space.com
Component : Graphic Tool

-----------------------------------------------------------------------

Purpose   : Graphic Tool based on GTK+ that provide several kinds of
            plot. This tool is to be used with the generic TSP protocol.

File      : Graphic Tool main part.

-----------------------------------------------------------------------
*/


/*
 * System includes.
 */
#include <stdio.h>
#include <assert.h>


/*
 * GDISP+ includes.
 */
#include "kernel.h"
#include "prototypes.h"


/*
 * GDISP+ main part.
 */
gint
main (int argc, char **argv) 
{

  Kernel_T *gdispKernel  = (Kernel_T*)NULL;


  /*
   * Set up GTK+ things, such as the default visual and color map.
   * Initialize the library for use, sets up default signal handlers, and
   * checks the arguments passed to the application on the command line.
   */
  gtk_init(&argc, &argv);


  /*
   * Allocate memory for managing GDISP+ kernel.
   * Take into account user defined arguments passed to the application
   * on the command line.
   */
  gdispKernel = gdispCreateKernel(argc,argv);


  /*
   * Create our colormap.
   */
  gdispCreateColormap(gdispKernel);


  /*
   * Create a graphic page and its 2D plot.
   */
  gdispCreateGraphicPage(gdispKernel);


  /*
   * Call the step function of the plot 2D system each <<period>>.
   */
  gdispKernel->timerPeriod   = 10; /* milli-seconds en theorie */
  gdispKernel->timerWatchDog = 1000 / gdispKernel->timerPeriod;
  gdispKernel->gdispTimeout  = gtk_timeout_add(gdispKernel->timerPeriod,
					       gdispPageStep,
					       (gpointer)gdispKernel);


  /*
   * Enter GTK main processing loop.
   * Sleep waiting for X events (such as button or key presses), timeouts,
   * file IO notifications to occur.
   */
  gtk_main();
  if (gdispKernel->gdispTimeout > 0) {

    gtk_timeout_remove(gdispKernel->gdispTimeout);
    gdispKernel->gdispTimeout = 0;

  }


  /*
   * End of GDISP+ main part, by destroying kernel.
   */
  gdispDestroyKernel(gdispKernel);
  return 0;

}
