/*!  \file 

$Id: kernel.c,v 1.1 2003/04/5 18:22:25 tsp_admin Exp $

-----------------------------------------------------------------------

GDISP+ - Graphic Tool based on GTK+,
         for being used upon the generic Transport Sampling Protocol.

Copyright (c) 2003 - Yves DUFRENNE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-----------------------------------------------------------------------

Project   : TSP
Maintainer: tsp@astrium-space.com
Component : Graphic Tool

-----------------------------------------------------------------------

Purpose   : Graphic Tool based on GTK+ that provide several kinds of
            plot. This tool is to be used with the generic TSP protocol.

File      : Graphic Tool Kernel Implementation.
            Kernel allocation.

-----------------------------------------------------------------------
*/

#ifndef __KERNEL_C__
#define __KERNEL_C__


/*
 * System includes.
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <pthread.h>
#include <errno.h>


/*
 * GDISP+ includes.
 */
#include "kernel.h"
#include "prototypes.h"


/*
 --------------------------------------------------------------------
                             STATIC ROUTINES
 --------------------------------------------------------------------
*/



/*
 --------------------------------------------------------------------
                             PUBLIC ROUTINES
 --------------------------------------------------------------------
*/


/*
 * Allocate memory for GDISP+ Kernel.
 */
Kernel_T*
gdispCreateKernel (gint    argc,
		   gchar **argv)
{

  Kernel_T *kernel = (Kernel_T*)NULL;


  /*
   * Memory allocation.
   */
  kernel = (Kernel_T*)g_malloc0(sizeof(Kernel_T));
  assert(kernel);


  /*
   * Initialise all fonts.
   */
  gdisp_loadFonts(kernel->fonts);


  /*
   * Initialise the plot system.
   */
  gdisp_initPlot2DSystem(kernel,&kernel->plotSystem);


  /*
   * Return the kernel itself.
   */
  return kernel;

}


/*
 * Free memory used by GDISP+ Kernel.
 */
void
gdispDestroyKernel (Kernel_T *kernel)
{

  assert(kernel);

  /*
   * Initialise all fonts.
   */
  gdisp_destroyFonts(kernel->fonts);

  /*
   * Destroy our colormap.
   */
  gdispDestroyColormap(kernel);

  /*
   * Free Kernel.
   */
  memset(kernel,0,sizeof(Kernel_T));
  g_free(kernel);

}



#endif /* __KERNEL_C__ */
