/*!  \file 

$Id: plot2D.c,v 1.1 2003/04/5 18:22:25 tsp_admin Exp $

-----------------------------------------------------------------------

GDISP+ - Graphic Tool based on GTK+,
         for being used upon the generic Transport Sampling Protocol.

Copyright (c) 2003 - Yves DUFRENNE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-----------------------------------------------------------------------

Project   : TSP
Maintainer: tsp@astrium-space.com
Component : Graphic Tool

-----------------------------------------------------------------------

Purpose   : Graphic Tool based on GTK+ that provide several kinds of
            plot. This tool is to be used with the generic TSP protocol.

File      : 2D plot system.

-----------------------------------------------------------------------
*/

#ifndef __PLOT_2D_C__
#define __PLOT_2D_C__


/*
 * System includes.
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <errno.h>
#include <math.h>


/*
 * GDISP+ includes.
 */
#include "kernel.h"
#include "plot2D.h"


/*
 --------------------------------------------------------------------
                             STATIC ROUTINES
 --------------------------------------------------------------------
*/

#define EVENT_METHOD(widget,event) \
        GTK_WIDGET_CLASS(GTK_OBJECT(widget)->klass)->event

#define X_WIN_TO_PLOT(plot,x) (x)
#define Y_WIN_TO_PLOT(plot,y) ((plot)->p2dAreaHeight - (y))

#define X_PLOT_TO_WIN(plot,x) (x)
#define Y_PLOT_TO_WIN(plot,y) ((plot)->p2dAreaHeight - (y))

#define Y_SYMBOL_OFFSET 15


/*
 * Copy back buffer into graphic area (front buffer).
 * C'est la methode qui copie le BACK BUFFER dans le FRONT BUFFER.
 * Tout simplement.
 *
 * A UTILISER DIFFEREMMENT PAR VOUS.
 *
 */
static void
gdisp_plot2DSwapBuffers (Kernel_T *kernel,
			 Plot2D_T *plot)
{

#if defined(DEBUG_2D)
  fprintf(stdout,"Swaping front and back buffers\n");
  fflush (stdout);
#endif

  /*
   * Copy back buffer into front buffer.
   */
  gdk_draw_pixmap(plot->p2dArea->window,
		  plot->p2dGContext,
		  plot->p2dBackBuffer,
		  0,
		  0,
		  0,
		  0,
		  plot->p2dArea->allocation.width,
		  plot->p2dArea->allocation.height);

}


/*
 * Redraw back buffer content.
 * C'est ici que je mets tout ce qu'il faut pour redessiner completement
 * le BACK BUFFER.
 *
 * Ce n'est evidemment pas optimise pour les perfos, car je commence par
 * effacer le contenu avant de dessiner pour de bon.
 *
 * Vous trouverez ici tout ce qu'il faut pour utiliser les couleurs et les
 * fontes, pour dessiner un rectangle et une chaine de caracteres.
 * Il y a meme de quoi faire evoluer les axes...
 * Pour le reste, regarder les APIs de la GLIB.
 *
 */
static void
gdisp_plot2DDrawBackBuffer ( Kernel_T *kernel,
			     Plot2D_T *plot )
{

#if defined(DEBUG_2D)
  fprintf(stdout,"Drawing into back buffer\n");
  fflush (stdout);
#endif

  static gint     xDirection    =   1;
  static gint     yDirection    =   1;
  static gint     xPosition     =  10;
  static gint     yPosition     =  10;
  static gint     xWidth        = 100;
  static gint     yHeight       = 100;
  static gint     pixelStep     =   5;
  static gint     angle         =   0;
  static gint     xRadius       =  50;
  static gint     yRadius       =  50;
  static gint     color         =  70;
  static GdkPoint fourPoints[4] = { { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 } };
  static gint     ruler         =   0;
  gboolean        updateRuler   = FALSE;

  /*
   * Clear back buffer.
   */
  gdk_gc_set_foreground(plot->p2dGContext,
			&kernel->colors[_BLACK_]);

  gdk_draw_rectangle(plot->p2dBackBuffer,
		     plot->p2dGContext,
		     TRUE, /* rectangle is filled */
		     0,
		     0,
		     plot->p2dArea->allocation.width,
		     plot->p2dArea->allocation.height);

  /*
   * Take care of direction.
   */
  xPosition += xDirection * pixelStep;
  yPosition += yDirection * pixelStep;
  angle     += (xDirection * yDirection);
  xRadius   += xDirection;
  yRadius   += yDirection;
  color     += (xDirection * yDirection);

  if (xPosition < 0) {
    xPosition   = 0;
    xDirection  = 1;
    ruler      += 10;
    updateRuler = TRUE;
  }
  if (yPosition < 0) {
    yPosition   = 0;
    yDirection  = 1;

  }
  if (xPosition + xWidth > plot->p2dArea->allocation.width) {
    xPosition   = plot->p2dArea->allocation.width - xWidth;
    xDirection  = -1;
    ruler      += 10;
    updateRuler = TRUE;
  }
  if (yPosition + yHeight > plot->p2dArea->allocation.height) {
    yPosition  = plot->p2dArea->allocation.height - yHeight;
    yDirection = -1;
  }
  if (xRadius < 0)
    xRadius = 0;
  if (yRadius < 0)
    yRadius = 0;
  if (color < 70)
    color = 70;
  if (color > 119)
    color = 119;

  fourPoints[0].x = plot->p2dArea->allocation.width  / 2;
  fourPoints[0].y = plot->p2dArea->allocation.height / 2;
  fourPoints[1].x = plot->p2dArea->allocation.width  / 2;
  fourPoints[1].y = plot->p2dArea->allocation.height / 2;
  fourPoints[2].x = plot->p2dArea->allocation.width  / 2;
  fourPoints[2].y = plot->p2dArea->allocation.height / 2;
  fourPoints[3].x = plot->p2dArea->allocation.width  / 2;
  fourPoints[3].y = plot->p2dArea->allocation.height / 2;

  fourPoints[0].x += xRadius * cos(3.14 * (double)(angle      ) / 180.0);
  fourPoints[0].y += yRadius * sin(3.14 * (double)(angle      ) / 180.0);
  fourPoints[1].x += xRadius * cos(3.14 * (double)(angle +  90) / 180.0);
  fourPoints[1].y += yRadius * sin(3.14 * (double)(angle +  90) / 180.0);
  fourPoints[2].x += xRadius * cos(3.14 * (double)(angle + 180) / 180.0);
  fourPoints[2].y += yRadius * sin(3.14 * (double)(angle + 180) / 180.0);
  fourPoints[3].x += xRadius * cos(3.14 * (double)(angle + 270) / 180.0);
  fourPoints[3].y += yRadius * sin(3.14 * (double)(angle + 270) / 180.0);

  gdk_gc_set_foreground(plot->p2dGContext,
			&kernel->colors[color]);

  gdk_draw_polygon(plot->p2dBackBuffer,
		   plot->p2dGContext,
		   TRUE, /* polygon is filled */
		   fourPoints,
		   4);

  gdk_gc_set_foreground(plot->p2dGContext,
			&kernel->colors[_BLUE_]);

  gdk_draw_rectangle(plot->p2dBackBuffer,
		     plot->p2dGContext,
		     TRUE, /* rectangle is filled */
		     xPosition,
		     yPosition,
		     xWidth,
		     yHeight);

  gdk_gc_set_foreground(plot->p2dGContext,
			&kernel->colors[_WHITE_]);

  gdk_draw_string(plot->p2dBackBuffer,
		  plot->p2dFont,
		  plot->p2dGContext,
		  plot->p2dArea->allocation.width  - xPosition,
		  plot->p2dArea->allocation.height - yPosition,
		  "TSP");

  if (updateRuler == TRUE) {

    gtk_ruler_set_range(GTK_RULER(plot->p2dHRuler),
			ruler,
			ruler + 10,
			0,
			10);

    gtk_ruler_set_range(GTK_RULER(plot->p2dVRuler),
			-ruler,
			-ruler + 10,
			0,
			10);

  }

  /*
   * Black lines around the graphic area.
   */
  gdk_gc_set_foreground(plot->p2dGContext,
			&kernel->colors[_BLACK_]);

  gdk_draw_line(plot->p2dBackBuffer,
		plot->p2dGContext,
		0,                                   /* x1 */
		0,                                   /* y1 */
		plot->p2dArea->allocation.width - 1, /* x2 */
		0);                                  /* y2 */

  gdk_draw_line(plot->p2dBackBuffer,
		plot->p2dGContext,
		plot->p2dArea->allocation.width  - 1,  /* x1 */
		0,                                     /* y1 */
		plot->p2dArea->allocation.width  - 1,  /* x2 */
		plot->p2dArea->allocation.height - 1); /* y2 */

}


/*
 * Treat 'expose' X event.
 * What shall I do when the graphic area has to be refreshed ?
 *
 * Voici maintenant le callback que je cable pour gerer les evenements
 * X de type EXPOSE. Pour des questions de performances X, je ne redessine
 * que la region qui est impactee par l'EXPOSE.
 * C'est le serveur X qui generera ce qu'il faut d'evenements EXPOSE pour
 * redessiner la totalite de la fenetre.
 *
 * POUR VOUS, vu que l'on de redessine que les 20% de droite, et que le reste
 * provient du back buffer, gerer l'EXPOSE revient a swapper les deux buffers
 * (ce qui effacera les 20% de droite du meme coup) puis a redessiner les 20%
 * restants a la main SI LE SAMPLING EST OFF.
 * Si il est ON, le STEP suivant dessine pour nous les 20% de droite.
 *
 * Si vous ne comprenez en rien ce que j'ai ecrit, faites comme bon vous
 * semble.
 */
static gboolean
gdisp_plot2DExpose (GtkWidget       *area,
		    GdkEventExpose  *event,
		    gpointer         data)
{

  Kernel_T *kernel = (Kernel_T*)data;
  Plot2D_T *plot   = (Plot2D_T*)NULL;

#if defined(DEBUG_2D)
  fprintf(stdout,"Expose event\n");
  fflush (stdout);
#endif

  /*
   * Graphic area has now to be repainted.
   */
  plot = (Plot2D_T*)gtk_object_get_data(GTK_OBJECT(area),
					"plotPointer");

  gdk_gc_set_clip_rectangle(plot->p2dGContext,
			    &event->area);

  gdisp_plot2DSwapBuffers(kernel,plot);

  gdk_gc_set_clip_rectangle(plot->p2dGContext,
			    (GdkRectangle*)NULL);

  return TRUE;

}


/*
 * Treat 'configure' X event.
 * What shall I do when the graphic area has to be re-configured ?
 *
 * C'est ici que l'on gere les changements de configuration, c'est-a-dire
 * les changements de POSITION dans la fenetre mere et les changements de
 * TAILLE dans cette meme fenetre.
 * Seuls les changements de taille nous impacte ici, car dans tous les cas,
 * notre widget est toujours en (0,0) dans son container parent.
 *
 * ATTENTION : une fenetre GTK (ou X) est retaillable.
 * Ce n'est pas vrai pour les PIXMAPS en memoire. Donc retailler le back
 * buffer revient a detruire le pixmap courant et a recreer un nouveau a
 * la nouvelle taille.
 */
static gboolean
gdisp_plot2DConfigure (GtkWidget         *area,
		       GdkEventConfigure *event,
		       gpointer           data)
{

  Kernel_T *kernel        = (Kernel_T*)data;
  Plot2D_T *plot          = (Plot2D_T*)NULL;

#if defined(DEBUG_2D)
  fprintf(stdout,"Configure event\n");
  fflush (stdout);
#endif

  /*
   * Don't know why, but GTK generates a 'configure' event when
   * showing widgets before returning into the main X loop. Strange.
   */
  if (event->width == 1 && event->height == 1) {

#if defined(DEBUG_2D)
    fprintf(stdout,"Configure event returns suddendly\n");
    fflush (stdout);
#endif
    return TRUE;

  }


  /*
   * Graphic area has now to be resized
   */
  plot = (Plot2D_T*)gtk_object_get_data(GTK_OBJECT(area),
					"plotPointer");

  plot->p2dAreaWidth  = event->width;
  plot->p2dAreaHeight = event->height;


  /*
   * Dereference back buffer if this one exist.
   */
  if (plot->p2dBackBuffer != (GdkPixmap*)NULL) {

    gdk_pixmap_unref(plot->p2dBackBuffer);

  }


  /*
   * Create a pixmap for double buffering.
   * Deduce depth from graphic area window.
   */
  plot->p2dBackBuffer = gdk_pixmap_new(plot->p2dArea->window,
				       plot->p2dAreaWidth,
				       plot->p2dAreaHeight,
				       -1 /* same as window */);

  assert(plot->p2dBackBuffer);


  /*
   * Refresh back buffer.
   */
  gdisp_plot2DDrawBackBuffer(kernel,plot);

  return TRUE;

}

#define FROZEN_RULER

/*
 * Create a '2D plot' by providing an opaque structure to the
 * caller. This opaque structure will be given as an argument to all
 * plot function. These functions remain generic.
 */
static void*
gdisp_createPlot2D (Kernel_T *kernel)
{

  Plot2D_T *plot = (Plot2D_T*)NULL;


  /*
   * Dynamic allocation.
   */
  plot = g_malloc0(sizeof(Plot2D_T));
  assert(plot);

  /*
   * Create a table : dimension 2 x 2, homogeneous.
   * Insert a graphic area and two rulers in it.
   */
  plot->p2dTable = gtk_table_new(2,
				 2,
				 FALSE);

  plot->p2dArea = gtk_drawing_area_new();

  gtk_table_attach(GTK_TABLE(plot->p2dTable),
		   plot->p2dArea,
		   1,
		   2,
		   0,
		   1,
		   GTK_EXPAND | GTK_FILL,
		   GTK_EXPAND | GTK_FILL,
		   0,
		   0 );

#if defined(FROZEN_RULER)
  gtk_widget_set_events(plot->p2dArea,
			GDK_POINTER_MOTION_MASK      |
			GDK_POINTER_MOTION_HINT_MASK   );
#endif

  gtk_signal_connect(GTK_OBJECT(plot->p2dArea),
		     "expose_event",
		     (GtkSignalFunc)gdisp_plot2DExpose,
		     (gpointer)kernel);

  gtk_signal_connect (GTK_OBJECT(plot->p2dArea),
		      "configure_event",
		      (GtkSignalFunc)gdisp_plot2DConfigure,
		      (gpointer)kernel); 

  gtk_object_set_data(GTK_OBJECT(plot->p2dArea),
		      "plotPointer",
		      (gpointer)plot);

  /*
   * The horizontal ruler goes on bottom.
   * As the mouse moves across the drawing area, a 'motion_notify_event'
   * is passed to the appropriate event handler for the ruler.
   */
  plot->p2dHRuler = gtk_hruler_new();

  gtk_ruler_set_metric(GTK_RULER(plot->p2dHRuler),
		       GTK_PIXELS);

  gtk_ruler_set_range(GTK_RULER(plot->p2dHRuler),
		      0,
		      10,
		      0,
		      10);

#if !defined(FROZEN_RULER)
  gtk_signal_connect_object(GTK_OBJECT(plot->p2dArea),
			    "motion_notify_event",
			    (GtkSignalFunc)EVENT_METHOD(plot->p2dHRuler,
							motion_notify_event),
			    GTK_OBJECT(plot->p2dHRuler));
#endif

#if defined(FROZEN_RULER)
  gtk_widget_set_sensitive(plot->p2dHRuler,
			   FALSE /* no sensitive */);
#endif

  gtk_table_attach(GTK_TABLE(plot->p2dTable),
		   plot->p2dHRuler,
		   1,
		   2,
		   1,
		   2,
		   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   GTK_FILL,
		   0,
		   0);
    
  /*
   * The vertical ruler goes on the left.
   * As the mouse moves across the drawing area, a 'motion_notify_event'
   * is passed to the appropriate event handler for the ruler.
   */
  plot->p2dVRuler = gtk_vruler_new();

  gtk_ruler_set_metric(GTK_RULER(plot->p2dVRuler),
		       GTK_PIXELS);

  gtk_ruler_set_range(GTK_RULER(plot->p2dVRuler),
		      -10,
		      0,
		      0,
		      10);

#if !defined(FROZEN_RULER)
  gtk_signal_connect_object(GTK_OBJECT(plot->p2dArea),
			    "motion_notify_event",
			    (GtkSignalFunc)EVENT_METHOD(plot->p2dVRuler,
							motion_notify_event),
			    GTK_OBJECT(plot->p2dVRuler));
#endif

#if defined(FROZEN_RULER)
  gtk_widget_set_sensitive(plot->p2dVRuler,
			   FALSE /* no sensitive */);
#endif

  gtk_table_attach(GTK_TABLE(plot->p2dTable),
		   plot->p2dVRuler,
		   0,
		   1,
		   0,
		   1,
		   GTK_FILL,
		   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   0,
		   0);

  /*
   * Create a graphic context specific to this plot.
   */
  plot->p2dGContext =
    gdk_gc_new(GTK_WIDGET(kernel->pageWindow)->window);

  plot->p2dFont     = kernel->fonts[GD_FONT_BIG][GD_FONT_FIXED];


  /*
   * Return the opaque structure.
   */
  return (void*)plot;

}


/*
 * Destroy a '2D' plot opaque structure.
 */
static void
gdisp_destroyPlot2D(Kernel_T *kernel,
		    void     *data)
{

  Plot2D_T *plot = (Plot2D_T*)data;

  /*
   * Now destroy everything.
   */
  gdk_gc_destroy    (plot->p2dGContext  );
  gdk_pixmap_unref  (plot->p2dBackBuffer);
  gtk_widget_destroy(plot->p2dArea      );
  gtk_widget_destroy(plot->p2dHRuler    );
  gtk_widget_destroy(plot->p2dVRuler    );
  gtk_widget_destroy(plot->p2dTable     );

  /*
   * Free opaque structure.
   */
  memset(plot,0,sizeof(Plot2D_T));
  g_free(plot);

}


/*
 * Record the parent widget.
 */
static void
gdisp_setPlot2DParent (Kernel_T  *kernel,
		       void      *data,
		       GtkWidget *parent)
{

  Plot2D_T *plot = (Plot2D_T*)data;

  /*
   * Remember my parent.
   */
  plot->p2dParent = parent;

}


/*
 * Record initial dimensions provided by the calling process.
 */
static void
gdisp_setPlot2DInitialDimensions (Kernel_T *kernel,
				  void     *data,
				  guint     width,
				  guint     height)
{

  Plot2D_T *plot = (Plot2D_T*)data;

  /*
   * Remember my initial width and height.
   */
  plot->p2dAreaWidth  = width;
  plot->p2dAreaHeight = height;

}


/*
 * Give back to the calling process the top level widget
 * in order to be inserted in a possible container for further
 * dynamic X management.
 */
static GtkWidget*
gdisp_getPlot2DTopLevelWidget (Kernel_T  *kernel,
			       void      *data)
{

  Plot2D_T *plot = (Plot2D_T*)data;

  return (GtkWidget*)plot->p2dTable;

}


/*
 * By now, the '2D plot' widgets are created, but not shown yet.
 * Show them here.
 */
static void
gdisp_showPlot2D (Kernel_T  *kernel,
		  void      *data)
{

  Plot2D_T *plot = (Plot2D_T*)data;


  /*
   * Now show everything.
   */
  gtk_widget_show(plot->p2dArea  );
  gtk_widget_show(plot->p2dHRuler);
  gtk_widget_show(plot->p2dVRuler);
  gtk_widget_show(plot->p2dTable );

  /*
   * These 'show' operations generate 'configure" events. Strange.
   */

}


/*
 * Record any incoming symbols.
 * The drop coordinate (0,0) is at the lower left corner of the graphic area.
 */
static void
gdisp_addSymbolsToPlot2D (Kernel_T *kernel,
			  void     *data,
			  GList    *symbolList,
			  guint     xDrop,
			  guint     yDrop)
{

  /*
   * Nothing here for the standalone version.
   */

}


/*
 * Broadcast all symbols.
 */
static GList*
gdisp_getSymbolsFromPlot2D (Kernel_T *kernel,
			    void     *data,
			    gchar     axis)
{

  /*
   * Nothing here for the standalone version.
   */
  return (GList*)NULL;

}


/*
 * Broadcast all symbols.
 */
static void
gdisp_stepOnPlot2D (Kernel_T *kernel,
		    void     *data)
{

  Plot2D_T *plot = (Plot2D_T*)data;

  /*
   * Put here what must be done at each step.
   */
  gdisp_plot2DDrawBackBuffer(kernel,plot);
  gdisp_plot2DSwapBuffers   (kernel,plot);

}


/*
 --------------------------------------------------------------------
                             PUBLIC ROUTINES
 --------------------------------------------------------------------
*/


void
gdisp_initPlot2DSystem (Kernel_T     *kernel,
			PlotSystem_T *plotSystem)
{

  /*
   * We must here provide all 'Plot2D' private functions
   * that remain 'static' here, but accessible from everywhere
   * via the kernel.
   */
  plotSystem->psCreate            = gdisp_createPlot2D;
  plotSystem->psDestroy           = gdisp_destroyPlot2D;
  plotSystem->psSetParent         = gdisp_setPlot2DParent;
  plotSystem->psGetTopLevelWidget = gdisp_getPlot2DTopLevelWidget;
  plotSystem->psShow              = gdisp_showPlot2D;
  plotSystem->psAddSymbols        = gdisp_addSymbolsToPlot2D;
  plotSystem->psGetSymbols        = gdisp_getSymbolsFromPlot2D;
  plotSystem->psSetDimensions     = gdisp_setPlot2DInitialDimensions;
  plotSystem->psStep              = gdisp_stepOnPlot2D;

  /*
   * This plot system is completely supported because all functions
   * have been provided.
   */
  plotSystem->psIsSupported = TRUE;

}


#endif /* __PLOT_2D_C__ */
