/*!  \file 

$Id: utils.c,v 1.1 2003/04/5 18:22:25 tsp_admin Exp $

-----------------------------------------------------------------------

GDISP+ - Graphic Tool based on GTK+,
         for being used upon the generic Transport Sampling Protocol.

Copyright (c) 2003 - Yves DUFRENNE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-----------------------------------------------------------------------

Project   : TSP
Maintainer: tsp@astrium-space.com
Component : Graphic Tool

-----------------------------------------------------------------------

Purpose   : Graphic Tool based on GTK+ that provide several kinds of
            plot. This tool is to be used with the generic TSP protocol.

File      : GDISP utilities.

-----------------------------------------------------------------------
*/

#ifndef __UTILITIES_C__
#define __UTILITIES_C__


/*
 * System includes.
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>


/*
 * GDISP+ includes.
 */
#include "kernel.h"
#include "prototypes.h"


/*
 * Font definitions.
 */

#define GD_NormalFont_14_ "-Adobe-Helvetica-Medium-R-Normal--14-*-*-*-*-*-*"
#define GD_ItalicFont_14_ "-Adobe-Helvetica-Medium-O-Normal--14-*-*-*-*-*-*"
#define GD_FixedFont_14_  "-Adobe-Courier-Medium-R-Normal--14-*-*-*-*-*-*"

#define GD_NormalFont_12_ "-Adobe-Helvetica-Medium-R-Normal--12-*-*-*-*-*-*"
#define GD_ItalicFont_12_ "-Adobe-Helvetica-Medium-O-Normal--12-*-*-*-*-*-*"
#define GD_FixedFont_12_  "-Adobe-Courier-Medium-R-Normal--12-*-*-*-*-*-*"

#define GD_NormalFont_10_ "-Adobe-Helvetica-Medium-R-Normal--10-*-*-*-*-*-*"
#define GD_ItalicFont_10_ "-Adobe-Helvetica-Medium-O-Normal--10-*-*-*-*-*-*"
#define GD_FixedFont_10_  "-Adobe-Courier-Medium-R-Normal--10-*-*-*-*-*-*"

#define GD_NormalFont_08_ "-Adobe-Helvetica-Medium-R-Normal--08-*-*-*-*-*-*"
#define GD_ItalicFont_08_ "-Adobe-Helvetica-Medium-O-Normal--08-*-*-*-*-*-*"
#define GD_FixedFont_08_  "-Adobe-Courier-Medium-R-Normal--08-*-*-*-*-*-*"

#define GD_DefaultFontName_  "fixed"


/*
 --------------------------------------------------------------------
                             STATIC ROUTINES
 --------------------------------------------------------------------
*/


/*
 * Take here a time stamp expressed in 'nanoseconds'.
 */
HRTime_T
gdisp_getHRTime (void)
{
  /*
   * Returned time is expressed in 'nanoseconds'.
   */
#if defined(BREIZ)

  return gethrtime();

#else

  struct timeval tp;

  gettimeofday(&tp,(void*)NULL);

  return (  (HRTime_T)tp.tv_sec  * G_GINT64_CONSTANT(1000000000)
	  + (HRTime_T)tp.tv_usec * G_GINT64_CONSTANT(      1000));

#endif

}


/*
 * Font loading.
 */
void
gdisp_loadFonts(GdkFont *fonts[GD_FONT_MAX_SIZE][GD_FONT_MAX_TYPE])
{

  gchar *fontNames[GD_FONT_MAX_SIZE][GD_FONT_MAX_TYPE] =
    { { GD_NormalFont_10_, GD_ItalicFont_10_, GD_FixedFont_10_ },
      { GD_NormalFont_12_, GD_ItalicFont_12_, GD_FixedFont_12_ },
      { GD_NormalFont_14_, GD_ItalicFont_14_, GD_FixedFont_14_ } };

  FontSize_T fontSize = GD_FONT_SMALL;
  FontType_T fontType = GD_FONT_NORMAL;

  /*
   * Back up to default font in case of problem.
   */
  for (fontSize=GD_FONT_SMALL; fontSize<GD_FONT_MAX_SIZE; fontSize++) {

    for (fontType=GD_FONT_NORMAL; fontType<GD_FONT_MAX_TYPE; fontType++) {

      fonts[fontSize][fontType] =
	gdk_font_load(fontNames[fontSize][fontType]);

      if (fonts[fontSize][fontType] == (GdkFont*)NULL)
	fonts[fontSize][fontType] = gdk_font_load(GD_DefaultFontName_);

    } /* loop on types */

  } /* loop on sizes */

}


/*
 * Font destruction.
 */
void
gdisp_destroyFonts(GdkFont *fonts[GD_FONT_MAX_SIZE][GD_FONT_MAX_TYPE])
{

  FontSize_T fontSize = GD_FONT_SMALL;
  FontType_T fontType = GD_FONT_NORMAL;

  /*
   * Destroy all fonts.
   */
  for (fontSize=GD_FONT_SMALL; fontSize<GD_FONT_MAX_SIZE; fontSize++) {

    for (fontType=GD_FONT_NORMAL; fontType<GD_FONT_MAX_TYPE; fontType++) {

      gdk_font_unref(fonts[fontSize][fontType]);

    } /* loop on types */

  } /* loop on sizes */

}

#endif /* __UTILITIES_C__ */
